/* * EMBL-EBI
* MSD Group
* VAMSAS Project
* VAMAV
*/
package uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping;

import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.PDBUtil;
import uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.exceptions.UnableToGroupSequencesException;
import uk.ac.ebi.msd.vamsas.vamav.exceptions.IncorrectVamavModuleInitialisation;
import uk.ac.ebi.msd.vamsas.vamav.session.VAMAVSession;
import uk.ac.ebi.msd.vamsas.vamav.util.data.PDBEntry;
import uk.ac.ebi.msd.vamsas.vamav.util.mapping.MappingManager;
import uk.ac.ebi.msd.vamsas.vamav.util.pool.StringBufferPool;


import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Properties;

/**
 *Group sequences per structure
 *
 * First, add sequences, with DB ref is provided ( addSequence methods)
 * then, 
 *storeGroup Method (To create atribute file and alignment files)
 *
 *Each time a sequence is added, it is added to the determined grou.
 * @version 1
 *@author <a href="mailto:pierre@ebi.ac.uk">Pierre MARGUERITE</a>
 */
public class SequenceGrouping_structure  extends SequenceGrouping //implements uk.ac.ebi.msd.vamsas.vamav.VamaVModule{
{
	
	
	//private final VAMAVLogger logger = new VAMAVLoggerImpl(this.getClass());
	protected VAMAVSession vamavSession = null;
	
	
	/**
	 *  
	 * Constructor of the  SequenceGrouping class, given a mapping manager 
	 *
	 * @param _mappingManager the mapping manager to use for mapping 
	 */
	  public SequenceGrouping_structure(  MappingManager _mappingManager)
	    	{
	    		super(_mappingManager);
	    	}
	  
	  /**
		 *  
		 * Constructor of the  SequenceGrouping class, given a mapping manager 
		 *
		 * @param _mappingManager the mapping manager to use for mapping 
		 * @param sessionID current session for which will be used the current sequence grouping.
		 */
		  public SequenceGrouping_structure(MappingManager _mappingManager, VAMAVSession session )
		    	{
		    		super(_mappingManager);
		    		this.logger.setSession(session.getSessionID());
		    		this.vamavSession = session;
		    	}
	
	/**
	 * Inits the Sequence Grouping sub module, with a list of configuration properties.
	 */
	public void init (Properties moduleProperties) throws IncorrectVamavModuleInitialisation
		{
			super.init(moduleProperties);
		}
    
    /**
     * Add a sequence for grouping.
     * 
     * @param sequence chain sequence to add
     * @param name name of the sequence to add
     * @return a list of parameters to call the AstexViewer
     * @throws UnableToGroupSequencesException if an error occurs during the adding.
     */
    public Hashtable<String, Object>  addSequence(String sequence, String name) throws UnableToGroupSequencesException
    	{
    		if( sequence == null || sequence.length()<1)
    			{
    				this.logger.sessionWarn("SequenceGrouping addSequence - no Sequence Provided. Can not add.");
    				return null;
    			}
    		if( name == null || name.length()<1)
    			{
    				this.logger.sessionWarn("SequenceGrouping addSequence - no name Provided. Can not add.");
    				return null;
    			}
    		ArrayList<PDBEntry> entries = null;
    		try
    			{
    				entries = this.getMapping().retrievePDBIDFromSequence(sequence);
    			}
    		catch (IncorrectVamavModuleInitialisation inE)
    			{
    				this.logger.sessionError("Sequence Grouping addSequence - error during mapping execution. Can not proceed.", inE);
    				throw new UnableToGroupSequencesException();
    			}
    		Hashtable<String, Object> parameters = null;
    		if (entries == null ) return null;
    		StringBuffer pdbIDs = StringBufferPool.getString();
    		for (int entryNumber = entries.size() -1; entryNumber>=0; entryNumber--)
    			{
    				PDBEntry entry = entries.get(entryNumber);
    				if (entry == null) continue;
    					
    				parameters = new Hashtable<String, Object>();
    				int molNumber = 0;
    				if( !parameters.containsValue(entry))
    					{
    						parameters.put("moleculeG"+molNumber,entry);
    						parameters.put("nameG"+molNumber, entry.getPdbID());
    						this.logger.sessionDebug("New molecule added "+entry);
    					
    						molNumber++;
    					}
    				PDBUtil util = new PDBUtil(this.vamavSession);
    				
    				ArrayList<PDBUtil.Chain> chains = util.GetChain(entry);
    				util = null;
    				if( chains != null)
    					{
							/*System.out.println(" chains found ");*/
    						for (int chainN = chains.size()-1; chainN>=0; chainN --)
    							{
    								PDBUtil.Chain chain =chains.get(chainN);
									if (chain == null)continue;
									/*System.out.println("chain "+chain.getName()+ " "+chain.getSequence());*/
									String groupName = chain.getName();
									int pos = groupName.indexOf("_");
									if( pos>0) groupName = groupName.substring(0,pos);
									this.addSequenceToGroup(groupName,chain.getSequence(),chain.getName()); 
									this.addStructureToGroup(groupName, entry.getPdbID());
    							}
    					}
    				else 
    					this.logger.sessionWarn("no chains found ");
    				String id = entry.getPdbID();
    				this.addSequenceToGroup(id, sequence, name);
    				if(pdbIDs.length()<1) pdbIDs.append(id);
    				else pdbIDs.append(", "+id);
    			}
    		parameters.put(this.pdbidsproperty,pdbIDs.toString());
			StringBufferPool.returnString(pdbIDs);
    		return parameters;
    	}
    
    /**
     * Add a sequence for grouping, with an associated accessionID and its source.
     * 
     * @param sequence chain sequence to add for grouping
     * @param name name of the sequence to add
     * @param accessionID accessionId associated to the sequence
     * @param source source of the accessionID
     * @return list of parameters to call the AstexViewer, if needed.  Null, otherwise.
     * @throws UnableToGroupSequencesException if an error occurs during the adding.
     */
    public Hashtable<String, Object> addSequence(String sequence, String name, String accessionID, String source) throws UnableToGroupSequencesException
    	{
    		
    		if( sequence == null || sequence.length()<1)
    			{
    				this.logger.sessionError("SequenceGrouping - addSequence: Can not add sequence / The sequence is unknown.");
    				return null;
    			}
    		if( name == null || name.length()<1)
    			{
    				this.logger.sessionError("SequenceGrouping - addSequence: Can not add sequence / The name is unknown.");
    				return null;
    			}
    		if( source == null || source.length()<1)
    			{
    				this.logger.sessionWarn("SequenceGrouping - addSequence: the accession ID source is not provided / try to determine PDBID from the sequence.");
    				return this.addSequence(sequence, name);
    			}
    		if( accessionID == null || accessionID.length()<1)
    			{
    				this.logger.sessionWarn("SequenceGrouping - addSequence: the accession ID is not provided / try to determine PDBID from the sequence.");
    				return this.addSequence(sequence, name);
    			}
    		Hashtable<String, Object> parameters =  null;
    		try
    			{
    				ArrayList <PDBEntry> entries = null;
    		
    				if("PDB".equals(source))
    					{
    						PDBEntry entry = this.getMapping().getPDBEntryFromPDBID(accessionID);
    						if( entry != null)
    							{
    								entries = new ArrayList<PDBEntry>(1);
    								entries.add(entry);
    							}
    					}
    				if ("UNIPROT".equalsIgnoreCase(source))
    					{
    						entries = this.getMapping().retrievePDBIDFromUniprotAC(accessionID);
    					}
    				if( entries == null || entries.size()<1) 
    					{
    						this.logger.sessionError("No associated PDB ID to the accessionAC "+ accessionID + " from "+source);
    						return this.addSequence(sequence, name);
    					}
    				//boolean notFound = false;
    				//hasAlignmentSequences = true;
    				parameters = new Hashtable<String, Object>();
			
			//currenty only use the first entry
			//for ( int entryNumber = entries.size()-1; entryNumber>=0; entryNumber --)
				//{
    				int molNumber = 0;
    				StringBuffer pdbIDs = null;
    				if (entries != null || entries.size()<1)
    					{
    						PDBUtil util = new PDBUtil(this.vamavSession);
    						for (int entryNumber = entries.size() -1; entryNumber >=0; entryNumber --)
    							{
    								PDBEntry entry =  entries.get(entryNumber);
    								parameters.put("moleculeG"+molNumber,entry);
    								parameters.put("nameG"+molNumber, entry.getPdbID());
    						
    								ArrayList<PDBUtil.Chain> chains = util.GetChain(entry);
    						
    								if( chains != null)
    									{
    										this.logger.sessionDebug(" chains found ");
							//		StringBuffer buff = StringBufferPool.getString();
    										for (int chainN = chains.size()-1; chainN>=0; chainN --)
    											{
    												PDBUtil.Chain chain = chains.get(chainN);
    												if (chain == null)continue;
    												String groupName = chain.getName();
    												int pos = groupName.indexOf("_");
    												if( pos>0) groupName = groupName.substring(0,pos);
    												this.addSequenceToGroup(groupName, chain.getSequence(),chain.getName());  
    												this.addStructureToGroup(groupName, entry.getPdbID());
    												chain = null;
    											}
    									}
    								else 
    									this.logger.sessionDebug("no chains found ");
    					
    								this.addSequenceToGroup(entry.getPdbID(), sequence, name);
    								if( pdbIDs == null) pdbIDs = StringBufferPool.getString();
    								if(pdbIDs.length()>0)pdbIDs.append(", "+entry.getPdbID());
    								else pdbIDs.append(entry.getPdbID());
    								entries.remove(entry);
    							}
    						util = null;
    						if(!"PDB".equals(source))parameters.put(this.pdbidsproperty,pdbIDs.toString());
    						StringBufferPool.returnString(pdbIDs);
    					}
    				
    			//}
    		/*catch (IncorrectVamavModuleInitialisation inE)
    			{
    				logger.AppliError("Sequence Grouping addSequence - error during mapping execution. Can not proceed.", inE);
    				throw new UnableToGroupSequencesException();
    			}*/
    			}
    		catch (Exception inE)
    			{
    				this.logger.sessionError("Sequence Grouping addSequence - error during mapping execution. Can not proceed.", inE);
    				throw new UnableToGroupSequencesException();
    			}
			return parameters;
    	}
	
    /**
     * Retrieves the directory to store files in.
     * @return the output directory for created file by the sequence grouping processing.
     */
    public String getOutputDirectory()
    	{
    		File sessionDir = this.vamavSession.getSessionDirectory(); 
    		if( sessionDir != null)
    			return sessionDir.getAbsolutePath();
    		return null; //should not happen, has directory file is attached during object creation. 
    	}

	/**
	 * Retrieves current sessionID
	 * @see uk.ac.ebi.msd.vamsas.vamav.documentmanagement.grouping.SequenceGrouping#getSessionID()
	 */
	@Override
	public String getSessionID()
		{
			
			return this.vamavSession.getSessionID();
		}
    
    
 }
